#!/usr/bin/perl
use strict;

# ExSite module installation script

print "\nThis script installs a dynamic content driver (DCD) into a website.

Continue? [y] ";
my $input = <STDIN>;
chomp $input;
exit if ($input && $input !~ /^y/i) ;

### part 1: are we in an ExSite CGI directory?

if (! -e "page.cgi" || ! -d "ExSite") {
    print "\nSorry, it does not look like you are currently in an ExSite
installation.  Please ensure that the current working directory is an
ExSite cgi-bin directory, and begin again.\n";
    exit 1;
}
if (! -d "Modules") {
    print "\nError: no Modules directory to install into.\n";
    exit 1;
}

### part 2: load ExSite

use ExSite::Config;
&exsite_init;

### part 3: find our module repository

my $basepath = "/home/exware/exsite-plugins";

print "\nWe will install a dynamic content module from your local module
repository.  This is a directory containing copies of all the dynamic 
content modules that are available to this system.  You must provide
the repository base path.

Base path to modules: [$basepath] ";
$input = <STDIN>;
chomp $input;
if ($input) { $basepath = $input; }
if (! -d $basepath) {
    print "Error: invalid directory: $basepath\n";
    exit 1;
}

### part 4: pick a module

opendir D, $basepath;
my @mod = readdir D;
closedir D;
print "\nThe available dynamic content modules are: \n\n";
foreach my $mod (sort @mod) {
#    next if (! -d $mod || $mod eq "CVS");
    next if ($mod =~ /^\./);
    print "\t$mod\n";
}
PICK_MODULE:
print "\nWhich module are you installing? ";
my $module = <STDIN>;
chomp $module;
my @modules = split /,\s*/,$module;
foreach my $m (@modules) {
    if (! -e "$basepath/$m") {
	print "Warning: $basepath/$module not found.\n";
	$m = undef;
    }
    if ($m && -e "_Modules/$m") {
	print "Warning: $module appears to already be installed.\n";
	$m = undef;
    }
}

### part 5: install method

print "\nInstall method ('copy' or 'symlink'): [symlink] ";
my $install_method = <STDIN>;
chomp $install_method;
if ($install_method ne "copy") { $install_method = "symlink"; }

### part 6: summarize

print "\nWe are going to install $module into ",`pwd`,
    "using the $install_method method.\nContinue? [y] ";
$input = <STDIN>;
chomp $input;
exit if ($input && $input !~ /^y/i) ;

### part 7: install

foreach my $m (@modules) {
    next if (! $m);
    &install_file("$basepath/$m/cgi/Modules/$m.pm","Modules",
		  $install_method);
    &install_file("$basepath/$m/html","../html/_Modules/$m",
		  $install_method);
}

sub install_file {
    my ($src,$dest,$method) = @_;
    if ($method eq "copy") {
	print "cp $src $dest\n";
	`cp -r $src $dest`;
	if (-e "$dest/CVS") { `rm -r $dest/CVS` };
    }
    else {
	print "ln -s $src $dest\n";
	`ln -s $src $dest`;
    }
}
