#!/usr/bin/perl
#-----------------------------------------------------------------------
# captcha.cgi - generate captcha information
#
# Usage:
#
#    captcha.cgi?XXXXXXX...     generate an image of the encoded string
#
#    captcha.cgi?key=YYY        generate a complete captcha
#
#    captcha.cgi?postkey=YYY    generate a passive captcha (posting key)
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Crypt;
use ExSite::Input;
use ExSite::Captcha;

if ($ENV{HTTP_USER_AGENT} =~ /msnbot|Googlebot|Yahoo/i) {
    # looks like a robot - don't even bother
    print "content-type:text/plain\n\n";
    print "Silly robot.";
    exit;
}

&exsite_init;

my $q = $ENV{QUERY_STRING};

if ($q =~ /^postkey=/) {
    # AJAX-generated passive captcha
    &make_post_key();
}
elsif ($q =~ /^key=/) {
    # AJAX-generated interactive (image) captcha
    &make_captcha();
}
elsif ($q) {
    # generate the captcha image itself
    &make_captcha_image();
}

# generate a post key for a passive captcha

sub make_post_key {
    my $in = new ExSite::Input;
    my $q = $in->query();
    my $c = new ExSite::Captcha;
    my $postkey = $c->get_post_key($q->{postkey});
    print "Content-type: text/plain\n\n";
    print "$postkey";
}

# generate a complete image captcha

sub make_captcha {
    my $in = new ExSite::Input;
    my $q = $in->query();
    # disallow text mode switing in AJAX mode
    my $c = new ExSite::Captcha(no_text_captcha=>1);
    my $captcha = $c->captcha_image($q->{key});
    print "content-type: text/html\n\n";
    print $captcha;
}

# generate the captcha image

sub make_captcha_image {
    my $c = new ExSite::Captcha;
    my $imgdata = $c->make_captcha_image($ENV{QUERY_STRING});
    print "content-type:image/png\n\n";
    print $imgdata;
}
