#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# ctrl-panel.cgi - generate a control panel for a web app
#
# Usage:  /cgi-bin/ctrl-panel.cgi/WEBAPP?options
#
# WEBAPP is the name of an application installed in the Modules area.
#
# All options are assumed to be directed at the Webapp, except:
# 
# _bare=1    Do not show the usual ExSite webtop wrapper.
#
#-----------------------------------------------------------------------
#
#   Copyright 2001-2008 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;  # use Form if inputting or processing forms
use ExSite::Module qw(get_module icon);
use ExSite::Input;
use ExSite::Misc;
use ExSite::ML;

use vars qw($db $query $ml $in $icon $name $info $title $out
	    $path @path $module $BARE);

&exsite_init;
$db = new ExSite::Form;
$ml = &get_obj("ML");

# process our inputs

$in = new ExSite::Input;
$query = $in->query();           # get query data
$BARE = $query->{_bare};         # BARE is usually true for popups and dialogs
$path = $in->path_info();        # get path info
@path = split/\//,$path;
$module = $path[1];              # 1st element of path is the Module
$in->path("module","/$module");  # define module path segment in case there is 
                                 # more path data
# get our content

if ($db->authorize > 1) {
    my $app = &get_module($module);
    if ($app) {
	$icon = &ExSite::Module::icon($module); #$app->icon;
	$name = $app->ioctl('ModuleName') || $module;
	$title = "$name - $config{site}{name}";
	my $panel = $app->ioctl('ControlPanel');
	if (ref $panel eq "CODE") {
	    $out .= ( eval { $app->$panel } || $@ );
	}
	else {
	    $out .= $ml->h1($msg{"Error executing $module control panel"});
	}
    }
    else {
	$icon = "$config{server}{HTMLpath}/_ExSite/images/exsite-button-round.gif";
	$title = $msg{"Error"};
	if ($@) {
	    $out .= $ml->h1(&substitute("Failed to start module [[module]]",
					{module=>$module})).$ml->p($@);
	}
	else {
	    $out .= $ml->h1(&substitute("[[module]]: Permission denied (no license)",{module=>$module}));
	}
    }
}
else {
    $out .= $ml->h1($msg{"Permission denied."}).
	$ml->p($msg{"You do not have permission to access this control panel."});
    $out .= $share{DB}->login_form(undef,$config{server}{server}.&relink());
#    $out .= $share{DB}->login_form(undef,"$config{prog}{ctrlpanel}/$module");
}

# initialize our page
$ml->Doctype($config{doctype});
$ml->Doc($out);

# wrap the content in some boilerplate HTML
if ($BARE) {
    # add some controls to the top
    $ml->Prepend(
		 $ml->div(
			  $ml->a("&times;&nbsp;&nbsp;close",{href=>"javascript:window.close()"}),
			  {class=>"close_button"} 
			  )
		 );
    # wrap with a body
    $ml->__body({class=>"ctrlpanel"});
    # add a head
    $ml->Prepend(
		 $ml->head(
			   $ml->title($title).
			   $ml->link(undef,
				     {rel=>"stylesheet",
				      type=>"text/css",
				      href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"})
			   )
		 );
}
else {
    my $help_button = (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/help.html") ?
	$ml->td(
		$ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-help.gif",height=>30,width=>30,border=>0,alt=>"Help",title=>"Help"}),
		       { href=>"javascript:popup_medium('$config{server}{HTMLpath}/_Modules/$module/help.html')" }
		       ),
		) : "";
    
    $ml->__div({class=>"ctrlpanel"});
    
    my $topbar = $ml->table(
			    $ml->tr(
				    $ml->td(
					    $ml->img(undef,{src=>$icon,height=>32,width=>32,border=>0,alt=>$name})
					    ).
				    $ml->td(
					    $ml->span($title,{class=>"title"}),
					    {class=>"titlebar"}
					    ).
				    $ml->td(
					    $ml->a(
						   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-home.gif",height=>30,width=>30,border=>0,alt=>"Home",title=>"Home"}),
						   { href=>"$config{server}{server}$config{server}{CGIpath}/$config{prog}{ctrlpanel}/$module" }
						   ),
					    ).
				    $help_button,
				    ),
			    {class=>"titlebar"}
			    );
    
    # masthead + controls
    $ml->Prepend($ml->div($topbar,{class=>"topbar"}));

    # wrap with a body
    $ml->__body({class=>"ctrlpanel",
		 marginheight=>0,marginwidth=>0,topmargin=>0,leftmargin=>0,
		 rightmargin=>0});
    
}

# CSS

my $css = $ml->link(undef,
		    {rel=>"stylesheet",
		     type=>"text/css",
		     href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"});
if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/$module.css") {
    # add a custom stylesheet
    $css .= $ml->link(undef,
		      {rel=>"stylesheet",
		       type=>"text/css",
		       href=>"$config{server}{HTMLpath}/_Modules/$module/$module.css"});
}

# Javascript

my $js = $ml->script(undef,
		     {type=>'text/javascript',
		      src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js"});
if ($config{jquery}) {
    # add standard jquery link
    $js .= $ml->script(undef,{type=>'text/javascript',src=>$config{jquery}});
};
if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/$module.js") {
    # add a custom JS file
    $js .= $ml->script(undef,
		     {type=>'text/javascript',
		      src=>"$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/$module.js"});
};

# add a head
$ml->Prepend(
	     $ml->head(
		       $ml->meta(undef,
				 {"http-equiv"=>"Content-Type",
				  content=>"text/html; charset=utf-8"}).
		       $ml->title($title).
		       $ml->link(undef,{rel=>"shortcut icon",href=>"$config{server}{HTMLpath}/_ExSite/images/exsite.ico"}).
		       $css.
		       $js
		       )
	     );
$ml->__html();
$ml->PrintWithHeader();

&exsite_close();

### END

