#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# dcd.cgi - call DCD directly
#
# DCDs (web applications, or plug-ins) are normally invoked through 
# special CMS tags that are processed using page.cgi.  In some cases
# we would like to call the DCDs directly, however;  dcd.cgi can be 
# used in this case.  This is normally used internally by ExSite --
# it would not normally be invoked directly from the browser.
#
# USAGE:
#    dcd.cgi/Module?dcd_query_data&_ajax=ID&_ref=referrer_url
#
# The referrer url is used to determine page and site position, in cases
# where the DCD needs a context to produce results.
#
# _ajax=ID directs the DCD to relink back into the DCD by using AJAX 
# (httpRequest) methods to fetch new DCD content using dcd.cgi, and
# inserting the fetched content into DOM element "ID".
#-----------------------------------------------------------------------
#
#   Copyright 2001-2006 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
# TODO:
# - allow for content expansion of the output
# - link inter-page services correctly
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Module;
use ExSite::Form;
use ExSite::Input;
use ExSite::URI;
use ExSite::Util;

use vars qw($db $dcdname $page_id $stat $dcd $license_ok 
	    $script $path $input $output $ref $query $prevquery);

&exsite_init;
if ($config{kill}) {
    # no dynamic content allowed
    $output = " ";
}
elsif ($config{busy} && (&sysload() > $config{busyload})) {
    # too busy to run the DCD
    $output = "Information not available at this time.";
}
elsif (($config{throttle}{user}{rate} || $config{throttle}{system}{rate}) && &throttle()) {
    # someone is hammering us
    $output = "Information not available at this time.";
}
else {
    my (undef,$dcdname) = split(/\//,$ENV{PATH_INFO});

    if ($dcdname) { 
	my $in = new ExSite::Input;
	my $q = $in->query;
	# httpRequest tools do not set the HTTP_REFERER, so we have to 
	# manually insert referer data into our URL
	my $ref = $q->{_ref};
	my $prog_re = join('|',values %{$config{prog}});
	
	### FIXME: this regexp does not work with PATH-based URLs
	$ref =~ /\/($prog_re)([^?\#]*)(\?(.*_id=(\d+).*))?/;
	$script = $1;
	$path = $2;
	$prevquery = $4;
	$page_id = $5;
	$db = new ExSite::Form;
	my $page;
	if ($page_id) {
	    $page = new ExSite::Page(id=>$page_id);
	}
	else {
	    $page = new ExSite::Page();
	    my ($pagerec,$pathtopage) = $page->path_to_page($path);
	    $page_id = (ref $pagerec) eq "HASH" ? $pagerec->{page_id} : 0;
	}
	
	# should we use AJAX for re-links, too?
	if ($q->{_ajax}) {
	    $share{ajax_target} = $q->{_ajax};
	}

	# set another language?
	my $version = $page->get_my("version");
	if ($version) {
	    (tied %msg)->version($version);
	}

	if ($ref) {
	    # change environment to make it look like we are at the reference URL
	    my $uri = &get_obj("URI");
	    $uri->setup($ref);
	    $uri->use_input;
	    $ENV{SCRIPT_NAME} =~ s/$config{prog}{dcd}$/$script/;
	    $ENV{SCRIPT_FILENAME} =~ s/$config{prog}{dcd}$/$script/;
	    $ENV{QUERY_STRING} = $uri->get("query");
	    $ENV{PATH_INFO} = $uri->get("path_info");
	    # reload the input buffers
	    delete $share{input};
	    my $in2 = new ExSite::Input;
	    $in2->query();
	    $in2->path_info();
	    # now add in any extra parameters that were passed to us
	    foreach my $param (keys %$q) {
		next if ($param =~ /^_/);   # skip dcd control parameters
		$in2->set($param,$q->{$param},"get");
	    }
	}

	# instantiate the reference page so that our module runs in proper context
	my $section;
	if ($page_id) {
	    my $page = new ExSite::Page(id=>$page_id);
	    $page->set_context("parent");
	    $page->fix_path();
	    $section = $page->get("section");
	    $share{Page} = $page;
	}
	
	# instantiate the module itself
	my $dcd = &ExSite::Module::get_module($dcdname,0,$section);
	if ($dcd) {
	    # execute the DCD method
	    my $options = $q->{_opt};
	    $output = $dcd->write($options) || $@;
	}
	else {
	    $output = $@ ? "$dcdname: $@" : "$dcdname: permission denied";
	}
    }
    else {
	$output = "DCD: no module!";
    }
}

if ($share{Page}) {
    # this gives us our content-type header, plus assorted
    # other headers (such as cookies)
    $share{Page}->show_headers();
    # do not show any other page output, however
    $share{Page}->cancel;
}
elsif (! $share{noContentHeader}) {
    # this situation (calling dcd.cgi with no page object) is most likely
    # to occur in some special AJAX applications.  In these cases, the 
    # plug-in can inform us that they've already issued their content-type
    # header by setting $share{noContentHeader}
    print "Content-type: text/html\n\n";
}
print $output;

