#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_image.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Dialog;
use ExSite::Input;

&exsite_init;

my $dlg = new ExSite::Dialog;
my $in = new ExSite::Input;
my %in = %{$in->combine()};
my $db = $dlg->db();
my $access = $db->authorize();

my $title = "Image/Document Tool"; # title
my $file = $config{prog}{dlg_image};
my $defaultimagedim = 200;
my $selectedlib = 0;

if (!$access){
	print "Location: $config{prog}{login}\n\n";
}elsif ($access < 2){
	my $out = &HTMLPageHeader();
	$out .= "<p>Access denied.  Webmasters only.</p>";
	$out .= &HTMLPageFooter();
	print $out;
}elsif ($in{action} eq "getimagelist"){
	# get image/document list
	print &getImageList(\%in);
}elsif ($in{action} eq "getimage"){
	# get single image/document
	print &getImage(\%in);
}elsif ($in{action} eq "geturl"){
	# get img url
	print &getUrl(\%in);
}else{
	if (!$in{section_id}) {
	    $in{section_id} = $dlg->find_section_id(\%in);
	}
	my $out = &HTMLPageHeader();
	# process image/document upload
	$out .= $in{action} eq "upload_file" ? &processUpload(\%in) : "";
	$out .= &PrintForm(\%in);
	$out .= &HTMLPageFooter();
	print $out;
}
#--------------------------------------------------------------------------------------
# Header
sub HTMLPageHeader {
	my (@links) = @_;
	my $css = &HTMLCSS();
	my $js = &HTMLJS;
	return <<END;
Content-type: text/html


<html>
<head>
<title>$title</title>
$css
$js
</head>
<body topmargin="0" leftmargin="0" style="border: 0px; margin: 0px;" scroll="no">
END
}
# Footer
sub HTMLPageFooter {
	return "\n</body>\n</html>";
}

# css style sheet
sub HTMLCSS{
	my $out = <<END;
<style>
	BODY { 
		background-color: buttonface; 
	}
	TABLE.dlg { 
		border: 0px; 
	}
	.dlg TD { 
		horizontal-align: left; height: 20px; 
	}
	.dlg INPUT { 
		border-size: 2px; 
	}
END
	# for msie
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= <<END;	
	TD, INPUT, SELECT { 
		font-family: "MS Sans Serif"; font-size: xx-small; 
	}
	INPUT.button { 
		border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
END
	}else{	
		$out .= <<END;
	TD, INPUT, P { 
		font-family: sans-serif; font-size: small; 
	}
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px;
	}
END
	}
	$out .= "\n</style>";
	return $out;
}
# javascript
sub HTMLJS{
	return <<END;
<script langauge="JavaScript" src="$config{server}{HTMLpath}/_ExSite/js/httprequest.js"></script>
<script language="JavaScript">
<!--
	var isImage = 0;
	function insertimage(){
		var objName,objAlign,objId,objUrl,re,tempImage;
		re = new RegExp('checker');
		tempImage = new Image();
		objName = getContentName(document.forms[0].obj_id.value);
		objId = getContentID(document.forms[0].obj_id.value);
		objUrl = getRequestData('$config{server}{CGIpath}/$file?action=geturl&content_id=' + objId);
		objAlign = document.forms[0].align.value;
		objNonImageContent = document.forms[0].nonimagecontent.checked == true ? 1 : 0;
		objImgBorderWidth = document.forms[0].imageborderwidth.value || 0;
		tempImage.src = objUrl;
		if (re.test(objUrl)){
			tempImage.width = 100;
			tempImage.height = 50;
		}
		objWidth = tempImage.width;
		objHeight = tempImage.height;
		window.close();
		window.opener.getImage(isImage,objName,objUrl,objAlign,objWidth,objHeight,objImgBorderWidth,objNonImageContent);
	}
	function cancel(){
		window.returnValue = null;
		window.close();
	}
	function showimage(){
		var target,image,requestUrl,re;
		re = new RegExp('img');
		target = document.getElementById('disp');
		id = getContentID(document.forms[0].obj_id.value);
		if (!id){
			return;
		}
		requestUrl = '$config{server}{CGIpath}/$file?action=getimage&content_id=' + id;
		image = getRequestData(requestUrl);
		if (re.test(image)){
			showAlignmentForm();
		}else{
			hideAlignmentForm();
		}
		target.innerHTML = image;
	}
	function getImageList(){
		var target,lib_id,requestUrl,list;
		hideAlignmentForm();
		clearPreviewBlock();
		lib_id = document.forms[0].lib_id.value;
		target = document.getElementById('imageSelector');
		requestUrl = '$config{server}{CGIpath}/$file?action=getimagelist&page_id=' + lib_id;
		list = getRequestData(requestUrl);
		target.innerHTML = list;
	}
	function hideAlignmentForm(){
		document.getElementById("imagealignment").disabled = true;
		document.getElementById("imagenoncontent").disabled = true;
		document.getElementById("imageborderwidth").disabled = true;
		document.getElementById("imagenoncontent").checked = true;
		isImage = 0;
	}
	function showAlignmentForm(){
		document.getElementById("imagealignment").disabled = false;
		document.getElementById("imagenoncontent").disabled = false;
		document.getElementById("imageborderwidth").disabled = false;
		document.getElementById("imagenoncontent").checked = false;
		isImage = 1;
	}
	function clearPreviewBlock(){
		var preview = document.getElementById('disp');
		preview.innerHTML = '';
	}
	function getContentID(val){
		var stringsplt = val.split(':');
		return stringsplt[0];
	}
	function getContentName(val){
		var result,stringsplt;
		result = '';
		stringsplt = val.split(':');
		for(var i = 1; i < stringsplt.length; i++){
			result += stringsplt[i];
		}
		return result;
	}
	function inputImgBorderWidth(obj){
		if (obj.checked == true){
			document.getElementById("imageborderwidth").disabled = true;
			document.getElementById("imagealignment").disabled = true;
		}else{
			document.getElementById("imageborderwidth").disabled = false;
			document.getElementById("imagealignment").disabled = false;
		}
	}
	function showImageOption(option){
		if (option == 'upload'){
			document.getElementById('image_insert').style.display = 'none';
			document.getElementById('image_upload').style.display = 'block';
		}else{
			document.getElementById('image_upload').style.display = 'none';
			document.getElementById('image_insert').style.display = 'block';
		}
	}
//-->
</script>
END
}
#--------------------------------------------------------------------------------------
# build insert image/document dialog
sub PrintForm{
	my $data = shift;
	# library drop down list
	my $library_selector= &getLibrarySelector($data);
	# image drop down list
	my $image_selector = &getImageSelector($data);
	# image alignment options
	my $image_attribute = &getAlignmentRadio($data);
	# preview area
	my $image_preview = &getPreviewBlock($data);
	
	my $upload_enable = $config{form}{htmleditor}{op_image_upload};

	my $todo_option = <<END;
<input type="radio" name="todo" value="insert" onclick="showImageOption(this.value);" checked>Select from a library
END
	my $image_option = <<END;
<div id="image_insert" style="display:block;">
<form onSubmit="insertimage()">
<table>
<tr>
	<td valign="top">
	<p>
	$library_selector
	</p>
	<p>
	$image_selector
	</p>
	<p>
	$image_attribute
	</p>
	<p>
	<input type="button" value="Insert" onClick="insertimage();">
	&nbsp;&nbsp;&nbsp;&nbsp;
	<input type="button" value="Cancel" onClick="cancel();">
	</p>
	</td>
	<td>&nbsp;&nbsp;</td>
	<td bgcolor="white" valign="top" width="150" align="center" style="border-left:solid;border-right:solid;border-top:solid;border-bottom:solid;">
	$image_preview
	</td>
</tr>
</table>
</form>
</div>
END
	# allow user to upload image/documents
	if ($upload_enable){
		$todo_option .= <<END;
<br>
<input type="radio" name="todo" value="upload" onclick="showImageOption(this.value);">Upload an image/document
END
		my $upload_form = &uploadForm($data);
		$image_option .= <<END;
<div id="image_upload" style="display:none;">
$upload_form
</div>
END
	}
	
	my $out = <<END;
<table cellspacing="10" border="0">
<tr>
	<td colspan="2"><b>$title</b></td>
</tr>
<tr>
	<td valign="top">Options</td>
	<td valign="top">$todo_option</td>
</tr>
<tr>
	<td valign="top">&nbsp;</td>
	<td valign="top">
	$image_option
	</td>
</tr>
END
	# set default library
	$out .= &setDefaultLib();
	return $out;
}

# document upload form
sub uploadForm{
	my $data = shift;
	my %data = %$data;
	require ExSite::Section;
	my $s = new ExSite::Section(id=>$data{section_id});
	my @libs = $s->fetch_libraries;
	my $select;
	foreach my $l (@libs){
		if ($db->owns($l->{section_id})){
			$select .= "\n<option value=\"$l->{page_id}\">$l->{filename}</option>";
		}
	}
	
	return <<END;
<form method="post" enctype="multipart/form-data" action="$config{server}{CGIpath}/$file">
<input type="hidden" name="action" value="upload_file">
<input type="hidden" name="section_id" value="$data{section_id}">
<input type="hidden" name="cont_id" value="$data{cont_id}">
<p>
Library:<br>
<select name="library">
<option value="">== Select ==</option>
$select		
</select>
</p>
<p>
File:<br>
<input type="file" name="fileupload">
</p>
<p>
<input type="submit" value="Upload">
&nbsp;&nbsp;&nbsp;&nbsp;
<input type="button" value="Cancel" onClick="cancel();">
</p>
</form>
END
}
# javascript popup
sub alertPopup{
	my ($msg) = @_;
	return <<END;
<script language="javascript">
<!--
alert('$msg');
// -->
</script>
END
}
# process form data
sub processUpload{
	my $data = shift;
	my %data = %$data;
	my $space = "&nbsp;&nbsp;";
	return &alertPopup("Error: File upload failed.  No library is selected.") if !$data{library};
	return &alertPopup("Error: File upload failed.  No file is selected.") if !$data{fileupload};
	my $filename;
	my $rawdata = undef;
	my $size = 0;
	my $mimedata = "";
	my $file = $data{fileupload};
	my $continue = 1;
	# read upload data
	while ($continue){
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	}
	return &alertPopup("Error: File upload failed.  No file data was received.") if !$size;
	my @path = split /[:\\\/]/, $file;        
        if ($path[-1]){
		$filename = ExSite::Misc::clean_filename($path[-1]);
	}
	my $fdata;
	if (ExSite::Misc::MimeType($filename) !~ /^image/i){
		require MIME::Base64;
		$fdata = $filename."#".MIME::Base64::encode_base64($rawdata);
	}else{
		require ExSite::Image;
		my $img = new ExSite::Image($filename,$rawdata);
		$img->shrink();
		$fdata = $img->encode;
	}
	require ExSite::Content;
	my $content = new ExSite::Content();
	# insert form data
	my $cid = $content->make_content({name=>$filename,page_id=>$data{library}},{fdata=>$fdata});
	if ($cid){
		return &alertPopup("$filename has been uploaded successfully.");
	}else{
		my @errors = $content->fetch_diagnostics("error");
		my $error_message = join("\\n",@errors);
		return &alertPopup($error_message);
	}
	return;
}
# create an image/document list
sub getImageList{
	my ($data) = @_;
	my %data = %$data;
	my $list = $db->fetch_match("content",{page_id=>$data{page_id}});	
	my $selector;
	foreach my $l (@$list){
		my $limit = 25;
		my $name = "$l->{name}";
		my $ext = "";
		if ($name =~ /(.+)\.(.+)$/){
			$name = $1;
			$ext = $2;
		}
		my $filename = $name;
		if (length($name) > $limit){
			$filename = substr($name,0,$limit)."...";
		}
		if ($ext){
			$filename .= ".${ext}";
		}
		$selector .= "\n<option value=\"$l->{content_id}:$l->{name}\">$filename</option>";
	}
	return <<END;
Expires: -1
Cache-Control: no-cache
Cache-Control: max-page=0
Pragma: no-cache
Content-Type: text/html


<select name="obj_id" onChange="showimage()">
<option value="">== Select ==</option>
$selector
</select>
END
}
sub getImage{
	my $data = shift;
	my %data = %$data;
	return $dlg->get_object("image",$data{content_id});
}
sub getUrl{
	my $data = shift;
	my %data = %$data;
	return $dlg->get_object("url",$data{content_id});
}
# create a library drop down list
sub getLibrarySelector{
	my $data = shift;
	my %data = %$data;
	my @libs = @{$dlg->fetch_library($data{section_id})};
	my $selector;
	foreach my $l (@libs){
		$selector .= "\n<option value=\"$l->{page_id}\"";
		# set first image library as the default
		if ($l->{filename} =~ /image/i && !$selectedlib){
			$selector .= " selected";
			$selectedlib = $l->{page_id};
		}
		$selector .= ">$l->{filename}</option>";
	}
	return <<END;
Library:<br>
<select id="lib_id" name="lib_id" onChange="getImageList()">
<option value="">== Select ==</option>
$selector
</select>
END
}
# create a default image/document drop down list
sub getImageSelector{
	return <<END;
Image/Document:<br>
<span id="imageSelector">
<select name="obj_id" onChange="showimage()">
<option value="">== Select ==</option>
</select>
</span>
END
}
# create an image alignment drop down list
sub getAlignmentRadio{	
	my @options = (
		{label=>"Left",value=>"left"},
		{label=>"Right",value=>"right"}
	);
	if ($config{form}{htmleditor}{imgalign_all}){
		my @extra = (
			{label=>"Top",value=>"top"},
			{label=>"Bottom",value=>"bottom"}
		);
		@options = (@options,@extra);
	}
	my $list_option;
	foreach my $opt(@options){
		$list_option .= <<END;
<option value="$opt->{value}">$opt->{label}</option>
END
	}
	return <<END;
<div id="img_alignment">
Image Properties:<br>
<table cellspacing="0">
<tr>
	<td>
<select id="imagealignment" name="align">
<option value="">None</option>
$list_option
</select>
	</td>
	<td>Alignment</td>
</tr>
<tr>
	<td><input type="text" id="imageborderwidth" name="imageborderwidth" value="0" size="2"></td>
	<td nowrap>Border Width</td>
</tr>
</table>
<p>
<input id="imagenoncontent" type="checkbox" name="nonimagecontent" value="1" onClick="inputImgBorderWidth(this)">Non-image Content
</p>
</div>
END
}
# preview area
sub getPreviewBlock{
	return <<END;
<b><center>Preview</center></b><br><div id="disp" width="200" height="600"></div>
END
}
# fetch image/document list for the default library
sub setDefaultLib{
	if ($selectedlib){
		return <<END;
<script language="javascript">
<!--
document.forms[0].lib_id.value = $selectedlib;
getImageList();
//-->
</script>
END
	}
	return;
}
### END
