#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_link.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use ExSite::Config;
use ExSite::Dialog;
use ExSite::Input;
&exsite_init();

my $dlg = new ExSite::Dialog;
my $in = new ExSite::Input();
my %data = %{$in->combine()};
my $file = $config{prog}{dlg_link};
my $db = $dlg->db();
my $access = $db->authorize;
my $OUT;
# dlg_mode:
#	1: include cms function
#	2: external link only
my $dlg_mode = $data{mode};

if ($access == 0) {
	print "Location: $config{prog}{login}\n\n";
} elsif ($access < 2) {
	$OUT .= &HTMLHeaderTop(&GetFormType());
	$OUT .= &HTMLHeaderBottom();
	$OUT .= "<p>Access denied (access level is $access).</p>";
	$OUT .= &HTMLFooter;
	print $OUT;
} elsif ($data{action} eq "getimagelist"){
	print &getImageList();
} elsif ($data{action} eq "checkfile"){
	print &checkValidFile($data{section},$data{name});
} elsif ($data{action} eq "getlib"){
	print &getlib($data{section_id},$data{name});
} elsif ($data{action} eq "geturl"){
	print &geturl($data{section_id},$data{url});
} elsif ($data{action} eq "getcontentname"){
	print &getcontentname($data{section_id},$data{url});
} else {
	$OUT .= &HTMLHeaderTop(&GetFormType());
	$OUT .= &HTMLPageJS(&FetchPage());
	#$OUT .= &HTMLHeaderJavascript();
	$OUT .= &HTMLHeaderBottom();
	my $html_header_js;
	if ($dlg_mode == 2){
		$OUT .= &PrintFormSimple(\%data);
		$html_header_js = &HTMLHeaderJavascriptSimple(\%data);
	}else{
		my @links = &GetSiteLinks(&FetchPage());
		$OUT .= &PrintForm(\%data,\@links);
		$html_header_js = &HTMLHeaderJavascript(\%data);
	}
	$OUT .= "<script language=\"Javascript\">${html_header_js}</script>";
	$OUT .= &HTMLFooter;
	print $OUT;
}

#-----------------------------------------------------------------------------------
sub getcontentname{
	my ($sectionid,$url) = @_;
	my $newurl = $url;
	$newurl =~ s/^.*?name=(.+?)(&(.*?))?$/$1/gis;
	return "content-type: text/html\n\n$newurl";
}
sub geturl{
	my ($sectionid,$url) = @_;
	my $newurl;
	{
		$url =~ /^.*?href=(\"|')?([^>]+?)(\s|\"|').*$/gis;
		$newurl = $2;
	}
	return "content-type: text/html\n\n$newurl";
}
sub getlib{
	my ($sectionid,$name) = @_;
	my $libid = 0;
	if ($sectionid && $name){
		require ExSite::Section;
		my $s = ExSite::Section->new(id=>$sectionid);
		$s->prefetch();
		my $c = $s->find($name);
		$libid = $c->get("content")->{page_id};
	}
	return "content-type: text/html\n\n$libid";
}
sub FetchPage{
	return $dlg->fetch_page(\%data);
}
sub GetFormType{
	return $data{url} ? "Edit" : "Insert";
}
sub checkValidFile{
	my ($sectionid,$filename) = @_;
	my $result = $dlg->is_exist($sectionid,$filename);
	return <<END;
content-type: text/html\n\n";
$result
END
}
sub GetSiteLinks {
	my $page_list = shift;
	my @links;
	foreach my $p (sort {uc($a->{label}) cmp uc($b->{label})} @$page_list){		
		my $url = "$p->{filename}";
		my $title = "$p->{label}";
		if (length($title) > 50) {
			$title = substr($title,0,50)."...";
		}
		$title .= " ($p->{filename})";
		# the rest go in order of the fetch, which is currently alphabetical
		push(@links,{url=>$url,text=>$title});
	}
	return @links;
}


# create a library drop down list
sub getLibrarySelector{
	my @libs = @{$dlg->fetch_library($data{section_id})};
	my $selector;
	foreach my $lib (@libs){
		my $name = $dlg->truncate_text($lib->{filename});
		# set first image library as the default
		$selector .= <<END;
<option value="$lib->{page_id}">$name</option>
END
	}
	return <<END;
<select id="lib_id" name="lib_id" onChange="getImageList()">
<option value="">== Select Library ==</option>
$selector
</select>
END
}
# Image/document drop down list
sub getImageList{
	my @content_list = $db->fetch_match("content",{page_id=>$data{page_id}});
	my $selector;
	foreach my $clist(@content_list){
		my $name = $dlg->truncate_text($clist->{name});
		$selector .= <<END;
<option value="$clist->{name}">$name</option>
END
	}
	return <<END;
content-type: text/html\n\n
<select name="obj_id" onChange="selectLibrary()">
<option value="">== Select Image/Document ==</option>
$selector
</select>
END
}
# Link text input
sub HTMLLinkText{
	my ($data) = @_;
	my $input_status = "";
	my $input_value = "highlighted_text";
	if ($data{title}){
		$input_status = "disabled";
	}else{
		$input_value = $data{title};
	}
	my $out = <<END;
<tr>
	<td width="100">Link Text</td>
	<td><input type="text" name="linkText" value="${input_value}" size="50" ${input_status}></td>
</tr>
END
	return $out;
}
# Link to inputs

sub HTMLLinkTo_CMS{
	my ($links) = @_;
	my @links = @$links;
	my $url = $data{url} || "http://";
	my $out = <<END;
<tr>
	<td>Link To</td>
	<td>
		<input type="radio" value="url" name="link_to" onFocus="linktoUpdate('url')" checked><span id="label_link_uri">URL</span><br>
		<input type="radio" value="page" name="link_to" onFocus="linktoUpdate('page')"><span id="label_link_page">Internal Page</span><br>
		<input type="radio" value="imgdoc" name="link_to" onFocus="linktoUpdate('imgdoc')"><span id="label_link_imgdoc">Image/Document</span>
	</td>
</tr>
<tr>	
	<td>&nbsp;</td>
	<td>
		<div id="div_internal_page" style="display: none;">
		<select name="linkSelect" onChange="selectMade()">
END
	if (scalar @links) {
		$out .= "<option value=\"\">==Select Internal Page ==</option>\n";
	} else {
		$out .= "<option value=\"\">==No Internal Pages Found==</option>\n";
	}
	foreach my $link (@links) {
		my $linktext = $dlg->truncate_text($link->{text});
		$out .= "<option value=\"$link->{url}\">$linktext</option>\n";
	}
	$out .= <<END;
		</select>
		</div>
		<div id="div_image_document" style="display:none;">
		<table>
		<tr>
			<td>
END
	$out .= &getLibrarySelector();
	$out .= <<END;
			</td>
			<td>
			<div id="imageSelector">
				<select name="obj_id" onChange="showimage()">
				<option value="">== Select Image/Document ==</option>
				</select>
			</div>
			</td>
		</tr>
		</table>
		</div>
		<div id="div_url" style="display: block;">
			<input type="text" id="link_url" name="linkURL" value="${url}" size="50" onChange="resetType()">
		</div>
	</td>
</tr>
END
	return $out;
}
# Print form
sub PrintForm {
	my ($data,$links) = @_;
	my @links = @$links;
	my $link_text = &HTMLLinkText($data);
	my $link_to = &HTMLLinkTo_CMS($links);
	my $type = &GetFormType();
	my $out = &HTMLImageListJS();
	$out .= <<END;
<div style="padding: 5px;">
<form id="dialog_form">
<table class="dlg">
<tr>
	<td colspan="2"><b>$type link</b></td>
</tr>
$link_text
<tr>
	<td colspan="2">&nbsp;</td>
</tr>
$link_to
</tr>
	<td>&nbsp;</td>
	<td><input type="checkbox" name="newwindow"> Open this link in a new window</td>
</tr>
<tr>
	<td colspan="2">&nbsp;</td>
</tr>
<tr>
	<td width="120">&nbsp;</td>
	<td>
		<input class="button" type="button" value="Submit" title="Submit" onclick="insertsubmit()">&nbsp;&nbsp;
		<input class="button" type="button" value="Cancel" title="Cancel Dialog" onclick="cancel()">
	</td>
</tr>
</table>
</form>
</div>
END
	return $out;
}
sub PrintFormSimple{
	my ($data) = @_;
	my $link_text = &HTMLLinkText($data);
	my $type = &GetFormType();
	my $out = &HTMLImageListJS();
	$out .= <<END;
<div style="padding: 5px;">
<form id="dialog_form">
<input type="hidden" name="link_to" value="url">
<table class="dlg">
<tr>
	<td colspan="2"><b>$type link</b></td>
</tr>
$link_text
<tr>
	<td colspan="2">&nbsp;</td>
</tr>
END
	my $url = $data{url} || "http://";
	$out .= <<END;
<tr>
	<td>Link To</td>
	<td>
		<input type="text" id="link_url" name="linkURL" value="${url}" size="50"
	</td>
</tr>
</tr>
	<td>&nbsp;</td>
	<td><input type="checkbox" name="newwindow"> Open this link in a new window</td>
</tr>
<tr>
	<td colspan="2">&nbsp;</td>
</tr>
<tr>
	<td width="120">&nbsp;</td>
	<td>
		<input class="button" type="button" value="Submit" title="Submit" onclick="insertsubmit()">&nbsp;&nbsp;
		<input class="button" type="button" value="Cancel" title="Cancel Dialog" onclick="cancel()">
	</td>
</tr>
</table>
</form>
</div>
END
	return $out;
}
#############################################################
sub HTMLFooter {
	return "\n</body></html>";
}
# Fetch a list of images/documents using ajax
sub HTMLImageListJS{
	return <<END;
<script langauge="JavaScript" src="$config{server}{HTMLpath}/_ExSite/js/httprequest.js"></script>
<script language="JavaScript">
<!--
	function getImageList(){
		var target,lib_id,requestUrl,list;
		lib_id = document.forms.dialog_form.lib_id.value;
		target = document.getElementById('imageSelector');
		requestUrl = '$config{server}{CGIpath}/$file?action=getimagelist&page_id=' + lib_id;
		list = getRequestData(requestUrl);
		target.innerHTML = list;
	}
//-->
</script>
END
}
# Create a list of webpages for look up purposes
sub HTMLPageJS{
	my $page_list = shift;;
	my $out = "var pageref = new Array();\n";
	foreach my $p(@$page_list){
		$out .= "pageref[\"$p->{filename}\"] = $p->{page_id};\n";
	}
	return $out;
}
sub HTMLHeaderTop{
	my $type = shift;
	my $out = <<END;
Content-type: text/html\n\n
<html>
<head>
<title>$type Link</title>
<link REL="stylesheet" TYPE="text/css" HREF="$config{server}{HTMLpath}/_ExSite/css/ExSite.css">
END
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= <<END;
<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: "MS Sans Serif"; font-size: xx-small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white; border-bottom: 1px solid black; border-right: 1px solid black;font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
</style>
END
	}else{
		$out .= <<END;
<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: sans-serif; font-size: small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button_style { border-top: 1px solid white; border-left: 1px solid white;border-bottom: 1px solid black; border-right: 1px solid black;font-size: x-small; width: 60px;}
</style>
END
	}
	$out .= <<END;
<script language="JavaScript">
<!--\n
END
	return $out;
}
sub HTMLHeaderBottom{	
	return <<END;
//-->
</script>
</head>
	<body topmargin="0" leftmargin="0" style="border: 0; margin: 0;" scroll="no">
END
}

sub HTMLHeaderJavascript{
	my ($in) = @_;
	my $enable_cms_link = $in->{section_id} ? 1 : 0;
	return <<END;
	var isImgDocObject = 0;
	var default_url = '';
	var default_page = '';
	var default_imgdoc = '';
	var default_lib = 0;
	var enable_cms_op = ${enable_cms_link};
	function attr(name, value) {
		if (!value || value == "") return "";
		return ' ' + name + '="' + value + '"';
	}
	function insertLink() {
		var linkname = document.forms.dialog_form.linkURL.value;
		var newWin = document.forms.dialog_form.newwindow.checked || 0;
		var linktext = document.forms.dialog_form.linkText.value;
		window.opener.getLink(linkname,linktext,newWin,isImgDocObject);
		window.close();
	}
	function cancel() {
		window.returnValue = null;
		window.close();
	}
	function selectMade() {
		var selector = document.forms.dialog_form.linkSelect;  
		var choice = selector.options[selector.selectedIndex];
		if (choice.value) {
			document.forms.dialog_form.linkURL.value = choice.value;
		}
	}
	function selectLibrary(){
		isImgDocObject = 1;
		var selector = document.forms.dialog_form.obj_id;
		var choice = selector.options[selector.selectedIndex];
		if (choice.value){
			document.forms.dialog_form.linkURL.value = choice.value;
		}
	}
	function urlsubmit() {
		var key = document.forms.dialog_form.linkURL.value;
		if (key){
			if (pageref[key]){
				document.forms.dialog_form.linkURL.value = pageref[key];
			}
		}
	}
	function insertsubmit(){
		urlsubmit();
		insertLink();
	}
	function resetType(){
		isImgDocObject = 0;
	}
	function linktoUpdate(val){
		var link_type = val || 'url';
		if (!enable_cms_op){
			link_type = 'url';
		}
		select_linkto_radio(link_type);
		switch(link_type){
		case 'imgdoc':
			document.getElementById('div_url').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'none';
			document.getElementById('div_image_document').style.display = 'block';
			document.forms.dialog_form.link_url.value = '';
			if (default_lib){
				for(var i = 0; i < document.forms.dialog_form.lib_id.length; i++){
					if (document.forms.dialog_form.lib_id.options[i].value == default_lib){
						document.forms.dialog_form.lib_id.selectedIndex = i;
					}
				}
			}
			getImageList();
			if (default_imgdoc){
				for(var i = 0; i < document.forms.dialog_form.obj_id.length; i++){
					if (document.forms.dialog_form.obj_id.options[i].value = default_imgdoc){
						document.forms.dialog_form.obj_id.selectedIndex = i;
					}
				}
			}
			link_type = 'imgdoc';
			break;
		case 'page':
			document.getElementById('div_url').style.display = 'none';
			document.getElementById('div_image_document').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'block';
			document.forms.dialog_form.link_url.value = '';
			if (default_page){
				for(var i = 0; i < document.forms.dialog_form.linkSelect.options.length; i++){
					if (document.forms.dialog_form.linkSelect.options[i].value == default_page){
						document.forms.dialog_form.linkSelect.selectedIndex = i;
					}
				}
			}
			link_type = 'page';
			break;
		default:
			document.getElementById('div_image_document').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'none';
			document.getElementById('div_url').style.display = 'block';
			document.forms.dialog_form.link_url.value = default_url || 'http://';
			break;
		}
	}
	
	function select_linkto_radio(radio_val){
		for(var i = 0; i < document.forms.dialog_form.link_to.length; i++){
			if (document.forms.dialog_form.link_to[i].value == radio_val){
				document.forms.dialog_form.link_to[i].checked = true;
			}else{
				if (!enable_cms_op){
					document.forms.dialog_form.link_to[i].disabled = true;
				}
				document.forms.dialog_form.link_to[i].checked = false;
			}
		}
		if (!enable_cms_op){
			var inactive_color = '#666666';
			document.getElementById("label_link_page").style.color = inactive_color;
			document.getElementById("label_link_imgdoc").style.color = inactive_color;
		}
	}
	function dialog_init(){
		var html_content;
		if (window.opener.isIE()){
			html_content = new String(window.opener.getEditorRange(window.opener.getEditorSelection()).htmlText);
		}else{
			var sel = window.opener.getEditorSelection();			
			var contents = window.opener.getEditorRange(sel).cloneContents();			
			var d = document.createElement('div');
			d.appendChild(contents);
			html_content = new String(d.innerHTML);
		}
		var clean_link_url = '$config{server}{server}$config{server}{CGIpath}/$file?action=geturl&seciton_id=$data{section_id}&url=' + escape(html_content);
		var link_url = getRequestData(clean_link_url);
		link_url = unescape(link_url);
		if (link_url.match(/content.cgi/i)){
			var ajax_imgdoc_url = '$config{server}{server}$config{server}{CGIpath}/$file?action=getcontentname&section_id=$data{section_id}&url=' + escape(link_url);
			var imgdoc_name = getRequestData(ajax_imgdoc_url);
			default_imgdoc = imgdoc_name;
			var fetch_lib_url = '$config{server}{server}$config{server}{CGIpath}/$file?action=getlib&section_id=$data{section_id}&name=' + default_imgdoc;
			default_lib = getRequestData(fetch_lib_url);
			linktoUpdate('imgdoc');
		}else if (link_url.match(/^http/i)){
			default_url = link_url;
			linktoUpdate('url');
		}else if (link_url.match(/^\\{\\{.+?\\}\\}/)){
			var cms_page = link_url;
			cms_page = cms_page.replace(eval('/^{{(.+?)}}\$/'),"\$1");
			var page_name;
			if (cms_page.match(/\\d+/)){
				for (var x in pageref){
					if (pageref[x] == cms_page){
						page_name = x;
						break;
					}
				}
			}else{
				page_name = cms_page;
			}
			if (page_name){
				default_page = page_name;
				linktoUpdate('page');
			}else{
				linktoUpdate('url');
			}
		}else{
			linktoUpdate('url');
		}
	}
	window.onload = dialog_init;
END
}
sub HTMLHeaderJavascriptSimple{
	my ($in) = @_;
	return <<END;
	var default_url = '';
	function attr(name, value) {
		if (!value || value == "") return "";
		return ' ' + name + '="' + value + '"';
	}
	function insertLink() {
		var linkname = document.forms.dialog_form.linkURL.value;
		var newWin = document.forms.dialog_form.newwindow.checked || 0;
		var linktext = document.forms.dialog_form.linkText.value;
		window.opener.getLink(linkname,linktext,newWin,0);
		window.close();
	}
	function cancel() {
		window.returnValue = null;
		window.close();
	}
	function urlsubmit() {
		var key = document.forms.dialog_form.linkURL.value;
		if (key){
			if (pageref[key]){
				document.forms.dialog_form.linkURL.value = pageref[key];
			}
		}
	}
	function insertsubmit(){
		urlsubmit();
		insertLink();
	}
	function linktoUpdate(val){
		var link_type = 'url';
		document.forms.dialog_form.link_url.value = default_url || 'http://';
		
	}
	function dialog_init(){
		var html_content;
		if (window.opener.isIE()){
			html_content = new String(window.opener.getEditorRange(window.opener.getEditorSelection()).htmlText);
		}else{
			var sel = window.opener.getEditorSelection();
			var contents = window.opener.getEditorRange(sel).cloneContents();
			var d = document.createElement('div');
			d.appendChild(contents);
			html_content = new String(d.innerHTML);
		}
		var clean_link_url = '$config{server}{server}$config{server}{CGIpath}/$file?action=geturl&seciton_id=$data{section_id}&url=' + escape(html_content);
		var link_url = getRequestData(clean_link_url);
		link_url = unescape(link_url);
		default_url = link_url;
		linktoUpdate('url');
	}
	window.onload = dialog_init;
END
}
### END
