#!/usr/bin/perl
#-----------------------------------------------------------------------
# echo.cgi - copy content to browser (utility used by WYSIWYG html editor)
#
# usage:
#
# echo.cgi?tab={TABLE}&col={COLUMN}&id={RECORD_ID}  # echo database field
# echo.cgi/path/to/file  # echo file in htdocs area
# echo.cgi?url={URL}  # echo remote URL
# echo.cgi    # echo std input
#
# extra arguments:
#  cms = if true, perform limited CMS substitutions on content
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;
use ExSite::ML;
use ExSite::Content;
use ExSite::Input;
use vars qw(%input $db);

&exsite_init;
my $in = new ExSite::Input;
my $query = $in->query;
$db = new ExSite::Form;
my $access = $db->authorize;

my $data;
if ($access >= 2 && ($query->{url} ||
		     ($query->{tab} && $query->{col} && $query->{id}) ||
		     $ENV{PATH_INFO})) { 
    $data = &get_html; 
}
elsif ($ENV{REQUEST_URI} =~ /$config{echo}{public_url}/) { 
    $data = &get_html; 
}

# now build a document

my $ml = new ExSite::ML;
$ml->Doctype($config{doctype});
$ml->_body($data,{class=>"HTMLeditor"});
$ml->Prepend(
	     $ml->head(
		       $ml->title("ExSite Editor").
		       $ml->meta(undef,{'http-equiv'=>"Content-Type",
					content=>"text/html; charset=$config{charset}"}).
		       $ml->link(undef,{rel=>"stylesheet", type=>"text/css", href=>"$config{server}{HTMLpath}/_ExSite/css/editor.css"})
		       )
	     );
$ml->__html();

print "Content-type: text/html\n\n";
$ml->Print;

# get contents

sub get_html {
    my $content;

    # case 1 : a file on the local machine

    if ($ENV{PATH_INFO}) {
	my $file = "$config{server}{HTMLroot}$config{server}{HTMLpath}$ENV{PATH_INFO}";
	#	 read file contents
	open F, "<$file";
	my $continue = 1;
	my $size = 0;
	while ($continue) {
	    $continue = read(F,$content,1024,$size);
	    $size += $continue;
	}
	close F;
    }

    # case 2 : a remote URL

    elsif ($query->{url}) {
	require LWP::UserAgent;
	# fetch the URL
	my $ua = new LWP::UserAgent;
	$ua->timeout(10);
	$ua->env_proxy;
	my $response = $ua->get($query->{url});
	if ($response->is_success) {
	    print $response->content;
	}
	else {
	    print $response->status_line;
	}
    }

    # case 3 : a field in the database

    elsif ($query->{tab} && $query->{col} && $query->{id}) {
	if (! $db->{map}->is_mapped($query->{tab})) {
	    return "Error - no such table as <b>$query->{tab}</b>";
	} 
	#if ($db->{map}->get($table,$column,"datatype") ne "html" ) {
	#	return "Error - column <b>$column</b> in table <b>$table</b> is not of type <b>html</b>";
	#} 
	if ($query->{cms} && $query->{tab} eq "content_data") {
	    # potential for CMS substitutions
	    require ExSite::ContentData;
	    my $cdata = new ExSite::ContentData(id=>$query->{id});
	    if ($cdata->content_type() eq "text/html") {
		# expand the content so it looks more realistic in the editor
		# - use dummy placeholders for content, modules so that we
		#   know how to convert them back to tags
		# - use real URL replacements, which look better, but use
		#   dynamic mode so we know how to convert them back to tags
		$content = $cdata->expand(
				 method=>"url,dummy-content,dummy-module",
				 mode=>"dynamic");
	    }
	}
	else {
	    if ($query->{id} =~ /^\d+$/) {
		my %data = $db->fetch($query->{tab},$query->{id});
		if (! exists $data{$query->{col}}) {
		    return "Error - cannot retrieve column <b>$query->{col}</b> in table <b>$query->{tab}</b> where <b>id=$query->{id}</b><br>You do not have permission to access this data.";
		} 
		$content = $data{$query->{col}};
	    }
	}
    }

    # case 4 : echo the data passed to us in the "data" parameter

    elsif ($query->{data}) {
	$content = $query->{data};
    }

    else {
	my $post = $in->post;
	if (ref $post eq "HASH") {

	    # case 5 : echo the POST data passed to us
	    $content = $post->{data};

	}
	else {

	    # case 6 : unknown

	    # WARNING: no data destination given;  action script must 
	    # know how to process the data.
	    $content = "";
	}
    }
    return $content;
}
