#!/usr/bin/perl
#-----------------------------------------------------------------------
# help.cgi - display help text 
#
# Usage: help.cgi/path/to/helpfile
#
# We look for the helpfile in 2 locations:  
# 1) CGIpath/dbmap/help
# 2) HTMLpath/_Modules/
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;

&exsite_init;

print "Content-type: text/html\n\n";

print "<HEAD>\n<TITLE>$config{site}->{name} Help</TITLE>\n";
print "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_ExSite/css/ExSite.css\">\n";
print "<style>
h1 { 
  background:url($config{server}{HTMLpath}/_ExSite/images/helptitlebg.jpg); 
  background-repeat:repeat-y; 
  padding:6px; padding-left:40px; 
}
body {
  padding:20px;
}
</style>\n";
print "</HEAD>\n<BODY>\n";

my $helpfile = $ENV{PATH_INFO};
$helpfile =~ s/^\///;

$helpfile =~ /^(dbmap\/help\/.*)$/;
my $safe_helpfile = $1;

my $helptext; 

if ($safe_helpfile) {
    # looks like a dbmap help file
    $helptext = &get_file($safe_helpfile);
    $helptext = "<p class=help>$helptext</p>\n";
}
else {
    $helpfile =~ /^(_Modules\/.*?\/*.html)$/;
    $safe_helpfile = $1;
    if ($safe_helpfile) {
	# looks like a module help file
	my $imgpath = $safe_helpfile;
	$imgpath =~ s/\/[^\/]*$//;
	$helptext = &get_file("$config{server}{HTMLroot}$config{server}{HTMLpath}/$safe_helpfile",{imgpath=>"$config{server}{HTMLpath}/$imgpath",href=>"$config{server}{HTMLpath}/$imgpath"});
    }
    else {
	# mystery help request - try to guess help topic

	if ($ENV{HTTP_REFERER}) {
	    $ENV{HTTP_REFERER} =~ /$config{prog}{ctrlpanel}\/(\w+)/;
	    $helpfile = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$1/help.html";
	    if (-e $helpfile) {
		# module-specific helpfile found
		$helptext = &get_file($helpfile,{imgpath=>"$config{server}{HTMLpath}/_Modules/$1"});
	    }
	    elsif ($ENV{HTTP_REFERER} =~ /$config{prog}{webdb}/) {
		$helpfile = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/WebDB/help.html";
		if (-e $helpfile) {
		    # use WebDB file
		    $helptext = &get_file($helpfile,{imgpath=>"$config{server}{HTMLpath}/_Modules/WebDB"});
		}
	    }
	}
	if (! $helptext && $config{help_url}) {
	    $helpfile = "$config{server}{HTMLroot}$config{help_url}";
	    if (-e $helpfile) {
		$helptext = "<p>No context-specific help found.</p>
<p><a href=$config{help_url}>Click here for an index of help topics.</a></p>";
	    }
	}
    }
}

print $helptext ? 
    $helptext : 
    "<p class=error>No help available on that topic.</p>\n";

print "<p align=center><a href=\"javascript:close()\" class=\"button\">close</a></p>\n";
print "</body>\n</html>\n";

### END

