#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# home.cgi - ExSite WebTop
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::PageBuf;
use ExSite::Misc;
use ExSite::HTML;
use ExSite::ML;
use ExSite::Module qw(module_list get_module icon);
use ExSite::Report;

use vars qw($db %query $popup_flag);

&exsite_init;
# connect to database
$db = new ExSite::DB;

my $ml = new ExSite::ML;
$ml->Doctype($config{doctype});

my $access = $db->authorize;
my $webtop_access = &webtop_access;
my $webtop = 0;
my %icongroup;

if ($webtop_access == 1) {

    # Web Top

    my @modules = ($access==2 && $config{webtop_whitelist}) ? 
	@{$config{webtop_whitelist}} :
	&module_list;
	
    my $errors;
    foreach my $modname (@modules) {
	my $mod = &get_module($modname);
	if ($mod) {
	    my $mod_access = $mod->ioctl('Access');
	    if (! defined $mod_access || $mod_access <= $access) {
		my $panel = $mod->ioctl('ControlPanel');
		if ($panel) {
		    # a control panel exists for this module;
		    my $iconurl = &icon($modname);
		    # find the control panel URL
		    my $url;
		    if (ref $panel eq "CODE") {
			$url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/$modname";
		    }
		    else {
			# assume ioctl has returned the control panel url
			$url = $panel;
		    }
		    # make an icon
		    my $icon = $ml->img(undef,{src=>$iconurl, height=>64, width=>64, border=>0, alt=>""});
		    $icon .= $ml->br();
		    $icon .= $mod->ioctl('ModuleName') || $modname;
		    # wrap icon in a link
		    my $link = $ml->a($icon,{class=>"webtop",href=>"javascript:popup_large('$url')"});
		    my $category = $mod->ioctl('Category') || "Applications";
		    # wrap linked icon in a wrapper for tiling
		    $icongroup{$category} .= $ml->div($link,{class=>"icon"});
		}
	    }
	}
	elsif ($@) {
	    $errors .= $ml->div($@,{class=>"error"});
	}
    }

    if ($errors && $config{debug}) {
	$icongroup{"Errors"} = $errors;
    }

    $ml->_div(
	      $ml->h1("Content Management").$icongroup{CMS},
	      { class=>"iconGroupA" }
	      );
    delete $icongroup{CMS};
    $ml->_div(
	      $ml->h1("Applications").$icongroup{Applications},
	      { class=>"iconGroupB" }
	      );
    delete $icongroup{Applications};
    $ml->_div(
	      $ml->h1("System").$icongroup{System},
	      { class=>"iconGroupA" }
	      );
    delete $icongroup{System};
    $ml->_div(
	      $ml->h1("Database").$icongroup{Database},
	      { class=>"iconGroupB" }
	      );
    delete $icongroup{Database};
    my $igrp = 0;
    foreach my $group (keys %icongroup) {
	$igrp++;
	$ml->_div(
		  $ml->h1($group).$icongroup{$group},
		  { class=> (($igrp % 2) ? "iconGroupA" : "iconGroupB") }
		  );
	delete $icongroup{group};
    }
}
elsif ($webtop_access == -1) {

    # the user has less than administrator access...

    # no webtop :-(

    require ExSite::Section;

    # 1) find out which websites they are connected to

    my %user = $db->my_user_record;
    my @sites = $db->my_real_sites;

    # TODO: if there is only one site, jump to it directly

    if (@sites == 0) {
	$ml->_h3("No Access");
	$ml->_p("Sorry, but you have not been granted management priveleges to any websites.  Please check with your system administrator.");
	$ml->_p($ml->a("&nbsp;&nbsp;Okay&nbsp;&nbsp;",{href=>"$config{server}{CGIpath}/$config{prog}{logout}", class=>"button"}));
    }
    elsif (@sites == 1) {
	my $s = new ExSite::Section(section=>$sites[0]);
	$s->set_context;
	my $url = $s->get_url;
	$ml->location($url);
    }
    else {
	$ml->_h2("$user{name}'s Websites");
	$ml->_p("You are associated with the following websites. Please select which one you want to open.");
	my @sitelist;
	foreach my $site (@sites) {
	    my $s = new ExSite::Section(section=>$site);
	    $s->set_context;
	    my $section = $s->get;
	    my $page_id = $s->id("page");
	    my $url;
	    if ($access == 2) {
		if ($page_id) {
		    $url = "$config{server}{CGIpath}/$config{prog}{page}?_id=$page_id&_cmsmode=1";
		    if (@sites == 1) {
			$ml->location($url);
		    }
		}
		else {
		    $url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/CMS?path=/$s->{data}{section_id}";
		    if (@sites == 1) {
			$ml->location($url);
		    }
		}
	    }
	    else {
		# no CMS interface for level 1 users
		$url = $s->httppath;
	    }
	
	    push @sitelist, { dt=> $ml->a($section->{title},{href=>$url}),
			      dd=> $section->{description} };
	}
	$ml->_dl(\@sitelist);
    }
    $ml->__div({style=>"padding:30px;"});
}
else {	
    $ml->_div($share{DB}->login_form());
}

$ml->__div({class=>"webtop"});

# logout action
my $logout = $config{auth}{logout_warning} ?
    &ExSite::HTML::Overlay(
			   pane=>$ml->h1("Log Out").
			   $ml->p("Continuing will log you out of the Administrator screens.").
			   $ml->p(&ExSite::HTML::Button(label=>"Log out",url=>"$config{server}{CGIpath}/$config{prog}{logout}")),
			   label=>$ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-exit.gif",height=>30,width=>30,border=>0,alt=>"Log out",title=>"Log out"}),
			   height=>150,
			   ) :
    $ml->a(
	   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-exit.gif",height=>30,width=>30,border=>0,alt=>"Log out",title=>"Log out"}),
	   { href=>"$config{server}{CGIpath}/$config{prog}{logout}" }
	   );

my $topbar = $ml->table(
		     $ml->tr(
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/exsite-button-round.gif",height=>30,width=>30,border=>0,alt=>"Home Page",title=>"Home Page"}),
					    { href=>"$config{server}{server}$config{server}{HTMLpath}" }
					    )
				     ).
			     $ml->td(
				     $ml->span($config{site}{name},{class=>"title"}),
				     {class=>"titlebar"}
				     ).
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-home.gif",height=>30,width=>30,border=>0,alt=>"Refresh Webtop",title=>"Refresh Webtop"}),
					    { href=>"$config{server}{CGIpath}/$config{prog}{home}" }
					    )
				     ).
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-help.gif",height=>30,width=>30,border=>0,alt=>"Help",title=>"Help"}),
					    { href=>$config{help_url}, target=>"help" }
					    )
				     ).
			     $ml->td($logout)
			     ),
		     {class=>"titlebar"}
		     );

$ml->Prepend($ml->div($topbar,{class=>"topbar"}));
$ml->__body({class=>"webtop"});
$ml->Prepend(
	     $ml->head(
		       $ml->title($config{site}{name}).
		       $ml->meta(undef,{'http-equiv'=>"Content-Type",
					content=>"text/html; charset=$config{charset}"}).
		       $ml->link(undef,{rel=>"shortcut icon",href=>"$config{server}{HTMLpath}/_ExSite/images/exsite.ico"}).
		       $ml->link(undef,{rel=>"stylesheet", type=>"text/css", href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"}).
		       $ml->script(undef,{type=>'text/javascript',src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js"})
		       )
	     );
$ml->__html;

print "Content-type: text/html\n\n";
$ml->Print;
&exsite_close;

# webtop_access
# returns -1 for denial to webtop only
#          0 for complete denial of access
#          1 for access to webtop
# customizable through handler

sub webtop_access {
    # use a handler in cases where simple access-level control is insufficient
    my $stat = $db->run_handler("webtop_access");
    return $stat if (defined $stat);
    # default to access level control
    my $access = $db->authorize;
    my $min_access = $config{webtop_access} || 3;
    if ($access) {
	return ($access >= $min_access) ? 1 : -1;
    }
    return 0;
}

### END
