#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# page.cgi - generate and display a web page
#
# USAGE:
#    page.cgi?_id=xxx
#    page.cgi/path/to/file.html
#
# Parameters:
#    _ = aggregated parameters (encrypted)
#    _id = id number of page to display
#    _cms = inlined editing flag (enable editing if true)
#    _cmsmode = persistent inlined editing flag (enable editing for all pages)
#    (other) = parameters for plug-in modules
#
# Other parameters are passed through to dynamic content objects that
# are embedded in the page.  By convention, parameters directed at page.cgi
# have a preceding underscore;  other parameters are intended for embedded
# dynamic content modules.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use ExSite::Config;
use ExSite::Form;
use ExSite::Page;
use ExSite::Session;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Input;

my ($utime1,$stime1,$cutime1,$cstime1) = times;
&exsite_init;

if ($config{kill}) {
    # gaah!  We've been killed!
    &kill_page();
}
elsif ($config{busy} && (&sysload() > $config{busyload})) {
    # we're swamped; delay the page
    &busy_page();
}
elsif (($config{throttle}{user}{rate} || $config{throttle}{system}{rate}) && 
       &throttle()) {
    # someone is hammering us; delay the page
    &throttle_page();
}
else {
    # normal operation
    &real_page();
}

&exsite_close;

# crude benchmarking tool for ExSite developers
# This measures pure code execution time, not Perl and library overhead.
my ($utime2,$stime2,$cutime2,$cstime2) = times;
if ($ENV{EXSITE_BENCHMARK}) {
    print "
User time:         ",($utime2-$utime1),"
System time:       ",($stime2-$stime1),"
Child User time:   ",($cutime2-$cutime1),"
Child System time: ",($cstime2-$cstime1),"\n";
}

# The kill page is displayed if the killswitch is tripped.  You can use this
# feature when performing system maintenance, or for disabling the
# most resource-intensive parts of a site for overload handling.

sub kill_page {
    my $killurl = $config{killurl} || "$config{server}{HTMLpath}/_ExSite/notavailable.html";
    print "Location: $killurl\n\n";
    return;
}

# The busy page is displayed if you have the busy switch enabled, and the
# 1 minute load average exceeds a certain threshold.  This serves primarily
# to limit the impact of DOS attacks and bad robots, but gives real readers
# a chance to reload.

sub busy_page {
    my $busyurl = $config{busyurl} || "$config{server}{HTMLpath}/_ExSite/serverbusy.html";
    print "Location: $busyurl\n\n";
    return;
}

# busy() decides if we are too busy to serve the real page.  This is the
# case if our 1 minute load average exceeds the value of $config{busyload}

sub busy {
    return (&sysload() > $config{busyload});
}

# The throttle page is displayed if you have the traffic throttler enabled, 
# and the request rate exceeds the allowed rate.  It enforces a wait until the
# request rate drops to an acceptable level.

sub throttle_page {
    my $busyurl = $config{throttle}{url} || "$config{server}{HTMLpath}/_ExSite/throttle.html";
    print "Location: $busyurl\n\n";
    return;
}

# show the real page...

sub real_page {
    my $db = new ExSite::Form;
    my $in = new ExSite::Input;
    my %query = %{$in->query};
    my $html;
    # create a null page
    my $p = new ExSite::Page;

    # check to see if we need embedded CMS features in this page;

    my $cmsflag = undef;
    my $access = $db->authorize;
    if ($access >= 2) {
	if (exists $query{_cms}) {
	    # _cms parameter can override all settings for this page only
	    $cmsflag = $query{_cms};
	}
	elsif (exists $query{_cmsmode}) {
	    # _cmsmode parameter can set mode for all pages
	    $cmsflag = $query{_cmsmode};
	    # turn inlined CMS features on/off for subsequent pages
	    my $cookieval = &get_cookie("ExSiteCMS");
	    if ($cmsflag != $cookieval) {
		# change the cookie value
		&edit_cookie("ExSiteCMS", $query{_cmsmode});  # this page
		$p->cookie("ExSiteCMS", $query{_cmsmode});    # subsequent pages
	    }
	}
	else {
	    # otherwise check if the mode was set previously
	    $cookie_setting = &get_cookie("ExSiteCMS");
	    if ($cookie_setting) { $cmsflag = $cookie_setting; }
	}
    }

    # generate the page

    my $got_page_id = 0;
    my %pagesetup = ( cms=>$cmsflag, preload=>1 );
    if ($query{_id}) {
	# /cgi/page.cgi?_id=XXX
	$pagesetup{id} = $query{_id};
	$got_page_id = 1;
    }
    elsif ($ENV{PATH_INFO}) {
	# /cgi/page.cgi/section/page.html
	$pagesetup{path} = 1;
	$got_page_id = 1;
    }
    if ($got_page_id) {
	# save page pointer for benefit of DCDs
	$share{Page} = $p;
	$p->setup( %pagesetup );
	$p->fix_path();   # in case we came in from the wrong URL type
	
	if ($p->is_active()) {
	    if ($p->has_content()) {
		# looks like we set up a real page

		# set the version/language
		my $version = $p->get_my("version");
		if ($version) {
		    # try to use matching system messages
		    (tied %msg)->version($version);
		}
		
		if ($p->authorize()) {
		    # insert all the content
		    $p->expand(cms=>$cmsflag);
		}
		else {
		    # permission denied!  Require user to log in before proceeding.
		    my $login_form;
		    my $postdata = $in->post;
		    if (keys %$postdata && exists $postdata->{login}) {
			# looks like a failed login attempt
			$login_form .= $p->show_diagnostics("error","html");
		    }
		    my $goto = $config{server}{server}.$ENV{REQUEST_URI};
		    $login_form .= $share{DB}->login_form("",$goto);
		    $p->errorpage($login_form);
		}
	    }
	    else {
		my $err = $msg{"Sorry, cannot build this page. (It appears to lack a template.)"};
		$p->errorpage($p->error($err));
	    }
	}
	else {
	    my $err = $msg{"Sorry, the page you have requested is not available at this time."};
	    $p->errorpage($p->error($err));
	}
    }
    else {
	my $err = $msg{"No page to display!"};
	$p->errorpage($p->error($err));
    }

    $p->show;
}

